@@ -6,7 +6,7 @@ saved, because of a ConnectionRefusedError/BrokenPipe/general OSErrors
 
 logzero logs all the exceptions, incase theyre not what I expect
 most of the times, mpv will be open for more than 10 minutes, so
-the WRITE_PERIOD periodically writes will at least capture what was
+the write period periodically writes will at least capture what was
 being listened to, even if *somehow* (has only happened
 when my computer suffers a random crash/shut-down)
 I lose data on what happened
@@ -15,16 +15,17 @@ BrokenPipes are captured in the event_eof function
 """
 
 import os
-import json
 import atexit
 from pathlib import Path
-from typing import List, Optional, Dict, Any
+from typing import List, Optional, Dict, Any, Type
 from time import sleep, time
 
 from python_mpv_jsonipc import MPV  # type: ignore[import]
 from logzero import logger, logfile  # type: ignore[import]
 
-SCAN_TIME: int = 10
+from .serialize import dump_json
+
+SCAN_TIME: int = int(os.environ.get("MPV_HISTORY_DAEMON_SCAN_TIME", 10))
 
 
 KNOWN_EVENTS = set(
@@ -47,9 +48,6 @@ KNOWN_EVENTS = set(
     ]
 )
 
-# every 10 minutes
-WRITE_PERIOD = 600
-
 
 def new_event(event_name: str, event_data: Any = None) -> Dict[str, Any]:
     """
@@ -86,7 +84,6 @@ class SocketData:
 
     Want to save:
         # at mpv launch:
-            # playlist
             # playlist-count
             # working-directory
             # is-paused
@@ -100,17 +97,28 @@ class SocketData:
                 # duration
     """
 
-    def __init__(self, socket: MPV, socket_loc: str, data_dir: str):
+    def __init__(
+        self,
+        socket: MPV,
+        socket_loc: str,
+        data_dir: str,
+        write_period: Optional[int] = None,
+    ):
         self.socket = socket
         self.socket_loc = socket_loc
         self.data_dir = data_dir
         self.socket_time = socket_loc.split("/")[-1]
         self.events: Dict[float, Dict] = {}
+        self.write_period = write_period if write_period is not None else 600
         # write every 10 minutes, even if mpv doesnt exit
-        self.write_at = time() + WRITE_PERIOD
+        self.write_at = time() + self.write_period
         # keep track of playlist/playlist-count, so we can use eof to determine
         # whether we should read next metadata
-        self.playlist_count = self.socket.playlist_count
+        pcount = self.socket.playlist_count
+        assert isinstance(
+            pcount, int
+        ), f"Playlist count is not an integer {self.socket_loc} {self.socket.playlist_count}"
+        self.playlist_count: int = pcount
         playlist_pos = self.socket.playlist_pos
         # incremented at the top of in store_file_metadata
         # technically this is zero-indexed, but have to deal with off-by-one errors
@@ -121,7 +129,7 @@ class SocketData:
             )
             self.playlist_index = 0
         else:
-            self.playlist_index = playlist_pos
+            self.playlist_index = int(playlist_pos)
         self.store_initial_metadata()
         self.store_file_metadata()
 
@@ -144,11 +152,12 @@ class SocketData:
 
     __str__ = __repr__
 
-    def write(self):
+    def write(self) -> None:
+        serialized = dump_json(self.events)
         with open(
             os.path.join(self.data_dir, f"{self.socket_time}.json"), "w"
         ) as event_f:
-            json.dump(self.events, event_f)
+            event_f.write(serialized)
 
     def nevent(self, event_name: str, event_data: Optional[Any] = None) -> None:
         """add an event"""
@@ -156,7 +165,7 @@ class SocketData:
         logger.debug(f"{self.socket_time}|{ct}|{event_name}|{event_data}")
         self.events[ct] = new_event(event_name, event_data)
 
-    def store_initial_metadata(self):
+    def store_initial_metadata(self) -> None:
         self.nevent("socket-added", time())
         self.nevent("working-directory", self.socket.working_directory)
         self.nevent("playlist-count", self.socket.playlist_count)
@@ -183,7 +192,7 @@ class SocketData:
         else:
             logger.warning(f"{self.socket_loc} Couldn't poll for {event_name}")
 
-    def store_file_metadata(self):
+    def store_file_metadata(self) -> None:
         """
         Called when EOF is reached (so, another file starts)
         """
@@ -222,7 +231,7 @@ class SocketData:
         """
         self.nevent("paused", {"percent-pos": self.socket.percent_pos})
 
-    def event_eof(self):
+    def event_eof(self) -> None:
         """
         Called when the 'eof' event happens. Doesn't necessarily mean mpv exits
         Could be going to the next song in the current playlist
@@ -240,16 +249,17 @@ class SocketData:
             if not isinstance(e, ConnectionRefusedError):
                 logger.exception(e)
 
-    def event_seeking(self):
+    def event_seeking(self) -> None:
         """
         Called when the user seeks in the file. Could possibly be called when a file is loaded as well
         """
-        if self.socket.percent_pos < 2:
+        pos = self.socket.percent_pos
+        if pos is not None and pos < 2:
             # logger.debug("ignoring seek because we just EOFd?")
             pass
         else:
             # save what % we seeked to
-            self.nevent("seek", {"percent-pos": self.socket.percent_pos})
+            self.nevent("seek", {"percent-pos": pos})
 
 
 class LoopHandler:
@@ -258,24 +268,35 @@ class LoopHandler:
     the main loop.
     """
 
-    def __init__(self, socket_dir: str, data_dir: str, autostart: bool = True):
+    def __init__(
+        self,
+        socket_dir: str,
+        data_dir: str,
+        *,
+        autostart: bool = True,
+        write_period: Optional[int],
+        socket_data_cls: Type[SocketData] = SocketData,
+    ):
         self.data_dir: str = data_dir
         self.socket_dir: str = socket_dir
+        self.write_period = write_period
         self._socket_dir_path: Path = Path(socket_dir).expanduser().absolute()
         self.sockets: Dict[str, MPV] = {}
+        self.socket_data_cls = socket_data_cls
         self.socket_data: Dict[str, SocketData] = {}
         if autostart:
             self.run_loop()
 
-    def scan_sockets(self):
+    def scan_sockets(self) -> None:
         """
         Look for any new sockets at socket_dir, remove any dead ones
         """
-        socket_loc: str = None
+        socket_loc_scoped: Optional[str] = None  # to access this after the try/except
         try:
             # iterate through all files
             for socket_name in os.listdir(self.socket_dir):
                 socket_loc: str = os.path.join(self.socket_dir, socket_name)
+                socket_loc_scoped = socket_loc
                 if socket_loc not in self.sockets:
                     # each of these runs in a separate thread, so the while loop below doesnt block event data
                     # ConnectionRefusedError thrown here
@@ -289,8 +310,8 @@ class LoopHandler:
                     if socket_loc in self.socket_data:
                         self.socket_data[socket_loc].socket = new_sock
                     else:
-                        self.socket_data[socket_loc] = SocketData(
-                            new_sock, socket_loc, self.data_dir
+                        self.socket_data[socket_loc] = self.socket_data_cls(
+                            new_sock, socket_loc, self.data_dir, self.write_period
                         )
                     self.attach_observers(socket_loc, new_sock)
                     self.debug_internals()
@@ -302,21 +323,22 @@ class LoopHandler:
             # this is probably unnecessary
             for s_loc, sock_obj in self.sockets.items():
                 # update higher scope to allow usage in except block
-                socket_loc = s_loc
+                socket_loc_scoped = s_loc
                 # try to access path to possibly cause ConnectionRefusedError,
                 # removing a dead socket
                 sock_obj.path
         except (ConnectionRefusedError, BrokenPipeError):
             logger.debug(
-                f"Connected refused for socket at {socket_loc}, removing dead/dangling socket file..."
+                f"Connected refused for socket at {socket_loc_scoped}, removing dead/dangling socket file..."
             )
             # make sure its actually removed from active sockets
             # gets removed from socket_data after 10 seconds
-            if socket_loc in self.sockets:
-                self.remove_socket(socket_loc)
+            if socket_loc_scoped is not None and socket_loc_scoped in self.sockets:
+                self.remove_socket(socket_loc_scoped)
             # rm -f
             try:
-                os.remove(socket_loc)
+                if socket_loc_scoped:
+                    os.remove(socket_loc_scoped)
             except FileNotFoundError:
                 pass
 
@@ -341,14 +363,14 @@ class LoopHandler:
         # if EOF next to seek, remove the seek
 
         @sock.property_observer("pause")
-        def on_pause(_name, value):
+        def on_pause(_, value):
             if value:  # item is now paused
                 socket_data.event_paused()
             else:
                 socket_data.event_resumed()
 
         @sock.property_observer("eof-reached")
-        def on_eof(_name, value):
+        def on_eof(_, value):
             # value == False means that eof has not been reached
             if isinstance(value, bool) and not value:
                 return
@@ -359,33 +381,39 @@ class LoopHandler:
             socket_data.event_eof()
 
         @sock.property_observer("seeking")
-        def on_seek(_name, value):
+        def on_seek(_, value):
             if isinstance(value, bool) and value:
                 socket_data.event_seeking()
 
     def remove_socket(self, socket_loc: str) -> None:
         if socket_loc in self.sockets:
             logger.debug(f"Removing socket {socket_loc}")
-            del self.sockets[socket_loc]
             # write quit event
-            self.socket_data[socket_loc].nevent("mpv-quit", time())
+            try:
+                self.socket_data[socket_loc].nevent("mpv-quit", time())
+            except KeyError:
+                pass
+            try:
+                del self.sockets[socket_loc]
+            except KeyError:
+                pass
         else:
             logger.warning(
                 "called remove socket, but socket_loc doesnt exist in self.sockets"
             )
         # (doesnt remove the file here, but should find it on the next scan_sockets call and remove it then)
 
-    def debug_internals(self):
-        logger.debug("sockets {}".format(self.sockets))
-        logger.debug("socket_data {}".format(self.socket_data))
+    def debug_internals(self) -> None:
+        logger.debug(f"sockets {self.sockets}")
+        logger.debug(f"socket_data {self.socket_data}")
 
-    def periodic_write(self):
+    def periodic_write(self) -> None:
         now = time()
-        for socket_loc, socket_data in self.socket_data.items():
+        for socket_data in self.socket_data.values():
             if now > socket_data.write_at:
                 logger.debug(f"{socket_data.socket_time}|running periodic write")
                 socket_data.write()
-                socket_data.write_at = now + WRITE_PERIOD
+                socket_data.write_at = now + socket_data.write_period
                 self.debug_internals()
 
     def write_data(self, force: bool = False) -> None:
@@ -413,8 +441,9 @@ class LoopHandler:
             for socket_data in self.socket_data.values():
                 socket_data.write()
 
-    def run_loop(self):
+    def run_loop(self) -> None:
         logger.debug("Starting mpv-history-daemon loop...")
+        logger.debug(f"Using socket class {self.socket_data_cls}")
         while True:
             self.scan_sockets()
             self.periodic_write()
@@ -422,17 +451,29 @@ class LoopHandler:
             sleep(SCAN_TIME)
 
 
-def run(socket_dir: str, data_dir: str, log_file: str) -> None:
+def run(
+    socket_dir: str,
+    data_dir: str,
+    log_file: str,
+    write_period: Optional[int],
+    socket_data_cls: Type[SocketData],
+) -> None:
     # if the daemon launched before any mpv instances
     if not os.path.exists(socket_dir):
         os.makedirs(socket_dir)
     assert os.path.isdir(socket_dir)
     os.makedirs(data_dir, exist_ok=True)
     assert os.path.isdir(data_dir)
-    logfile(log_file, maxBytes=1e7, backupCount=1)
-    l = LoopHandler(socket_dir, data_dir, autostart=False)
+    logfile(log_file, maxBytes=int(1e7), backupCount=1)
+    lh = LoopHandler(
+        socket_dir,
+        data_dir,
+        autostart=False,
+        write_period=write_period,
+        socket_data_cls=socket_data_cls,
+    )
     # incase user keyboardinterrupt's or this crashes completely
     # for some reason, write data out to files in-case it hasn't
     # been done recently
-    atexit.register(lambda: l.write_data(force=True))
-    l.run_loop()
+    atexit.register(lambda: lh.write_data(force=True))
+    lh.run_loop()
